-------------------------------------------------
---------------- DCS MovingMap ------------------
-------------------------------------------------
-------- ver.16 (23.04.24) by Bergison ----------
-------- https://movingmap.bergison.com ---------
-------------------------------------------------


function movingmap_load()

    package.path = package.path .. ";.\\LuaSocket\\?.lua;"
    package.cpath = package.cpath .. ";.\\LuaSocket\\?.dll;"
    
    local mm_socket = require("socket")
    local mm_host = "192.168.0.22" -- "127.0.0.1"
    local mm_port = 50000
	local mm_options_view
	local missionOptions
    local movingmap = {}
	local units_sent = 0
	local ok_to_send = 1
	local lTime
	local mmo = {}

    function movingmap.onSimulationStart()
		net.log("[MovingMap] broadcast started")
	
		local optionsData = _G["OptionsData"];
		local mm_input = optionsData.getPlugin("MovingMap", "mmHost");
		
		if mm_input ~= "" then
			mm_host = mm_input
		end
		
		missionOptions = DCS.getMissionOptions()
		mm_options_view = missionOptions["difficulty"]["optionsView"]
			
		mm = mm_socket.udp()
		mm:setoption('broadcast', true)
		mm:settimeout(1)  -- .001
		
		lTime = DCS.getModelTime()
    end

    function movingmap.onSimulationStop()
		if mm then
			mm_socket.try(mm:sendto("close_socket", mm_host, mm_port)) -- to close the listener socket
			mm:close()
		end
		net.log("[MovingMap] broadcast stopped")
    end
    
    function movingmap.onSimulationFrame()
		-- send data every x seconds, if previous sendloop is complete
		if ok_to_send == 1 and lTime + .1 < DCS.getModelTime() then
		
			-- stop sending new data (until sendloop is complete)
			ok_to_send = 0
		
			-- reset frame counter
			lTime = DCS.getModelTime()			
			
			if Export.LoGetSelfData() then
				
				-- send own_data
				local own_id = Export.LoGetPlayerPlaneId()
				local own_data = Export.LoGetSelfData()
				local own_expendables = Export.LoGetSnares()
				local own_wind = Export.LoGetVectorWindVelocity()
				local own_time = Export.LoGetModelTime()
				local own_IAS = Export.LoGetIndicatedAirSpeed()
				mm_packet = string.format("O,%d,%s,%d,%f,%f,%f,%f,%s,%s,%d,%f,%f,%f,%d,%d,%.3f,%f,%s",
												own_id,									-- 1 Unit ID (unique)
												own_data.Name,							-- 2
												own_data.Type.level2,					-- 3
												own_data.LatLongAlt.Lat, 				-- 4 Lat
												own_data.LatLongAlt.Long,				-- 5 Long
												own_data.LatLongAlt.Alt, 				-- 6 ALT
												own_data.Heading,						-- 7 HDG
												own_data.UnitName,						-- 8 Unit Name
												own_data.GroupName,						-- 9 Group Name
												own_data.CoalitionID,					-- 10 CoalitionID (1 or 2)
												own_wind.x, own_wind.y, own_wind.z,		-- 11 to 13 Wind vector at ownship
												own_expendables.chaff,					-- 14 Chaff
												own_expendables.flare,					-- 15 Flare
												own_time,								-- 16 Model time
												own_IAS,								-- 17 Indicated Airspeed
												mm_options_view							-- 18 Mission Export Setting
												)
				mm_socket.try(mm:sendto(mm_packet, mm_host, mm_port))
								
				-- send route <- DOES NOT WORK WITH MOST DCS MODELS :(
				-- local route = Export.LoGetRoute()
				--  if route then
				-- 	for num,wpt in pairs(route.route) do
						--lat,lon  = LoLoCoordinatesToGeoCoordinates(wpt.world_point.x,wpt.world_point.z);
				-- 		local mm_packet = string.format("R, %d, %f, %f, %f, %d",
				-- 										wpt.this_point_num,		-- point_num
				-- 										wpt.world_point.x,		-- wpt_pos x converted to lat
				-- 										wpt.world_point.y,		-- wpt_pos y
				-- 										wpt.world_point.z,		-- wpt_pos z converted to lon
				-- 										wpt.next_point_num		-- next wp
				-- 										)
				-- 		socket.try(mm:sendto(mm_packet, mm_host, mm_port))
				-- 	end
				-- end
				
				-- get all DCS units, if units list is empty (= all units have been sent)
				if next(mmo) == nil then
					mmo = Export.LoGetWorldObjects()
					units_sent = 0
				end	
				
				-- loop through all units
				for k,v in pairs(mmo) do
				
					mm_packet = nil
				
					-- format values for UNITS
					if v.GroupName ~= nil then
						mm_packet = string.format("A,%d,%s,%d,%.6f,%.6f,%f,%f,%s,%s,%d",  
														k,						-- 1 Unit ID (unique)
														v.Name,					-- 2
														v.Type.level2,			-- 3
														v.LatLongAlt.Lat, 		-- 4 Lat
														v.LatLongAlt.Long,		-- 5 Long
														v.LatLongAlt.Alt, 		-- 6 ALT
														v.Heading,				-- 7 HDG
														v.UnitName,				-- 8 Unit Name
														v.GroupName,			-- 9 Group Name
														v.CoalitionID			-- 10 CoalitionID (1 or 2)
														)
					
					-- format values for MISSILES
					elseif v.Type.level1 == 4 and v.Type.level2 == 4 then
						mm_packet = string.format("W,%d,%s,%.6f,%.6f,%f,%f,%d",
												k,								-- 1 Unit ID (unique)
												v.Name,							-- 2
												v.LatLongAlt.Lat, 				-- 3 Lat
												v.LatLongAlt.Long,				-- 4 Long
												v.LatLongAlt.Alt, 				-- 5 ALT
												v.Heading,						-- 6 HDG
												v.CoalitionID					-- 7 CoalitionID (1 or 2)
												)
					end
					
					-- send the next x number of UNITS or MISSILES, break loop if this number of units has been sent
					if mm_packet ~= nil then
						if units_sent < 30 then
							mm_socket.try(mm:sendto(mm_packet, mm_host, mm_port))
							units_sent = units_sent + 1
						else
							units_sent = 0
							break
						end	
					end
					
					-- remove the just sent unit from units list
					mmo[k] = nil
					
				end				
			end
			
			-- sendloop completeded, ready for new data
			ok_to_send = 1
			
		end 
	end
    
    DCS.setUserCallbacks(movingmap)

    net.log("[MovingMap] Loaded...")
end

local status, err = pcall(movingmap_load)
if not status then
    net.log("[MovingMap] Load Error: " .. tostring(err))
end